'use strict';

const Sequelize = require('./index');
const sequelize = require('./test/support').createSequelizeInstance();

const Op = Sequelize.Op;

let Model, ModelB, ModelC, ModelD, ModelE;

/**
 * @param {boolean} useEnum
 */
async function test(useEnum) {
  Model = sequelize.define('Model', { not_id: Sequelize.INTEGER }, {});
  ModelB = sequelize.define('ModelB', { model_id: Sequelize.INTEGER }, {});
  ModelC = sequelize.define('ModelC', { modelB_id: Sequelize.INTEGER }, {});
  ModelD = sequelize.define('ModelD', { modelC_id: Sequelize.INTEGER, modelE_id: Sequelize.INTEGER }, {});
  ModelE = sequelize.define('ModelE', {}, {});

  Model.hasMany(ModelB, { foreignKey: 'model_id' });
  ModelB.belongsTo(Model, { foreignKey: 'model_id' });

  ModelB.hasMany(ModelC, { foreignKey: 'modelB_id' });
  ModelC.belongsTo(ModelB, { foreignKey: 'modelB_id' });

  ModelC.hasMany(ModelD, { foreignKey: 'modelC_id' });
  ModelD.belongsTo(ModelC, { foreignKey: 'modelC_id' });

  ModelD.belongsTo(ModelE, { foreignKey: 'modelE_id' });
  ModelE.hasMany(ModelD, { foreignKey: 'modelE_id' });

  await sequelize.sync({ force: true });
  await populate();

  // console.log('Find that does not reproduce the issue')
  // const t1 = await Model.findOne({
  //   where: { id: 1 }, include: [
  //     {
  //       model: ModelB,
  //       required: true,
  //       include: [
  //         {
  //           model: ModelC,
  //           required: true,
  //           include: [
  //             {
  //               model: ModelD,
  //               required: true,
  //               include: [
  //                 {
  //                   model: ModelE,
  //                   required: true
  //                 }
  //               ]
  //             }
  //           ]
  //         }
  //       ]
  //     }
  //   ]
  // });

  console.log('Find that reproduces the issue')
  const t2 = await Model.findOne({
    logging: console.log,
    where: { not_id: 2 }, include: [
      {
        model: ModelB,
        required: true,
        include: [
          {
            model: ModelC,
            required: true,
            include: [
              {
                model: ModelD,
                required: true,
                include: [
                  {
                    model: ModelE,
                    required: true
                  }
                ]
              }
            ]
          }
        ]
      }
    ]
  });

  await sequelize.close();
}

async function populate() {
  const model = await Model.create({ not_id: 2 });
  const modelB = await ModelB.create({ model_id: model.id });
  const modelC = await ModelC.create({ modelB_id: modelB.id });
  const modelE = await ModelE.create({});
  const modelD = await ModelD.create({ modelC_id: modelC.id, modelE_id: modelE.id });
}

async function main() {
  try {
    await test();
  } catch (err) {
    console.error(err.message);
    process.exit(1);
  }
}

main();